@@ -56,6 +56,7 @@ def write_default_ansible_cfg(work_dir,
         '~/.ansible/plugins/callback:'
         '/usr/share/ansible/plugins/callback:'
         '%s/callback_plugins' % constants.DEFAULT_VALIDATIONS_BASEDIR)
+    callbacks_whitelist = ','.join(['profile_tasks'])
     roles_path = ('%(work_dir)s/roles:'
                   '/root/.ansible/roles:'
                   '/usr/share/ansible/roles:'
@@ -73,7 +74,10 @@ def write_default_ansible_cfg(work_dir,
     config.set('defaults', 'roles_path', roles_path)
     config.set('defaults', 'library', modules_path)
     config.set('defaults', 'callback_plugins', callbacks_path)
+    config.set('defaults', 'callback_whitelist', callbacks_whitelist)
     config.set('defaults', 'lookup_plugins', lookups_path)
+    # suppress tasks if all hosts skip, was previously full_skip callback
+    config.set('defaults', 'display_skipped_hosts', 'no')
 
     log_path = os.path.join(work_dir, 'ansible.log')
     config.set('defaults', 'log_path', log_path)
@@ -86,6 +90,15 @@ def write_default_ansible_cfg(work_dir,
     config.set('defaults', 'timeout', '30')
     config.set('defaults', 'gather_timeout', '30')
 
+    # Setup fact cache to improve playbook execution speed
+    config.set('defaults', 'gathering', 'smart')
+    config.set('defaults', 'fact_caching', 'jsonfile')
+    config.set('defaults', 'fact_caching_connection',
+               '/var/lib/mistral/ansible_fact_cache')
+
+    # Expire facts in the fact cache after 7200s (2h)
+    config.set('defaults', 'fact_caching_timeout', '7200')
+
     # mistral user has no home dir set, so no place to save a known hosts file
     config.set('ssh_connection', 'ssh_args',
                '-o UserKnownHostsFile=/dev/null '
