@@ -21,12 +21,35 @@ import requests
 import shutil
 
 from oslo_log import log as logging
+from tripleo_common.utils import image as image_utils
 
 LOG = logging.getLogger(__name__)
 
 
 IMAGE_EXPORT_DIR = '/var/lib/image-serve'
 
+MEDIA_TYPES = (
+    MEDIA_MANIFEST_V1,
+    MEDIA_MANIFEST_V1_SIGNED,
+    MEDIA_MANIFEST_V2,
+    MEDIA_MANIFEST_V2_LIST,
+) = (
+    'application/vnd.docker.distribution.manifest.v1+json',
+    'application/vnd.docker.distribution.manifest.v1+prettyjws',
+    'application/vnd.docker.distribution.manifest.v2+json',
+    'application/vnd.docker.distribution.manifest.list.v2+json',
+)
+
+TYPE_KEYS = (
+    TYPE_KEY_URI,
+    TYPE_KEY_TYPE
+) = (
+    'URI',
+    'Content-Type'
+)
+
+TYPE_MAP_EXTENSION = '.type-map'
+
 
 def make_dir(path):
     if os.path.exists(path):
@@ -61,40 +84,48 @@ def export_stream(target_url, layer, layer_stream, verify_digest=True):
     make_dir(blob_dir_path)
     blob_path = os.path.join(blob_dir_path, '%s.gz' % digest)
 
-    LOG.debug('export layer to %s' % blob_path)
+    LOG.debug('[%s] Export layer to %s' % (image, blob_path))
 
     length = 0
     calc_digest = hashlib.sha256()
 
     try:
         with open(blob_path, 'wb') as f:
+            count = 0
             for chunk in layer_stream:
+                count += 1
                 if not chunk:
                     break
+                LOG.debug('[%s] Writing chunk %i for %s' %
+                          (image, count, digest))
                 f.write(chunk)
                 calc_digest.update(chunk)
                 length += len(chunk)
+                LOG.debug('[%s] Written %i bytes for %s' %
+                          (image, length, digest))
     except Exception as e:
-        write_error = 'Write Failure: {}'.format(str(e))
+        write_error = '[{}] Write Failure: {}'.format(image, str(e))
         LOG.error(write_error)
         if os.path.isfile(blob_path):
             os.remove(blob_path)
-            LOG.error('Broken layer found and removed: %s' % blob_path)
+            LOG.error('[%s] Broken layer found and removed %s' %
+                      (image, blob_path))
         raise IOError(write_error)
     else:
-        LOG.info('Layer written successfully: %s' % blob_path)
+        LOG.info('[%s] Layer written successfully %s' % (image, blob_path))
 
     layer_digest = 'sha256:%s' % calc_digest.hexdigest()
-    LOG.debug('Provided layer digest: %s' % digest)
-    LOG.debug('Calculated layer digest: %s' % layer_digest)
+    LOG.debug('[%s] Provided digest: %s, Calculated digest: %s' %
+              (image, digest, layer_digest))
 
     if verify_digest:
         if digest != layer_digest:
             hash_request_id = hashlib.sha1(str(target_url.geturl()).encode())
             error_msg = (
-                'Image ID: %s, Expected digest "%s" does not match'
+                '[%s] Image ID: %s, Expected digest "%s" does not match'
                 ' calculated digest "%s", Blob path "%s". Blob'
                 ' path will be cleaned up.' % (
+                    image,
                     hash_request_id.hexdigest(),
                     digest,
                     layer_digest,
@@ -113,41 +144,73 @@ def export_stream(target_url, layer, layer_stream, verify_digest=True):
         )
         if blob_path != expected_blob_path:
             os.rename(blob_path, expected_blob_path)
+            blob_path = expected_blob_path
 
     layer['digest'] = layer_digest
     layer['size'] = length
-    return layer_digest
+    LOG.debug('[%s] Done exporting image layer %s' % (image, digest))
+    return (layer_digest, blob_path)
 
 
-def cross_repo_mount(target_image_url, image_layers, source_layers):
-    for layer in source_layers:
-        if layer not in image_layers:
-            continue
-
-        image_url = image_layers[layer]
-        image, tag = image_tag_from_url(image_url)
-        dir_path = os.path.join(IMAGE_EXPORT_DIR, 'v2', image, 'blobs')
-        blob_path = os.path.join(dir_path, '%s.gz' % layer)
-        if not os.path.exists(blob_path):
-            LOG.debug('Layer not found: %s' % blob_path)
-            continue
-
-        target_image, tag = image_tag_from_url(target_image_url)
-        target_dir_path = os.path.join(
-            IMAGE_EXPORT_DIR, 'v2', target_image, 'blobs')
-        make_dir(target_dir_path)
-        target_blob_path = os.path.join(target_dir_path, '%s.gz' % layer)
-        if os.path.exists(target_blob_path):
-            continue
-        LOG.debug('Linking layers: %s -> %s' % (blob_path, target_blob_path))
+def layer_cross_link(layer, image, blob_path, target_image_url):
+    target_image, _ = image_tag_from_url(target_image_url)
+    target_dir_path = os.path.join(
+        IMAGE_EXPORT_DIR, 'v2', target_image, 'blobs')
+    make_dir(target_dir_path)
+    target_blob_path = os.path.join(target_dir_path, '%s.gz' % layer)
+    if not os.path.exists(target_blob_path):
+        LOG.debug('[%s] Linking layers: %s -> %s' %
+                  (image, blob_path, target_blob_path))
         # make a hard link so the layers can have independent lifecycles
         os.link(blob_path, target_blob_path)
 
 
+def cross_repo_mount(target_image_url, image_layers, source_layers,
+                     uploaded_layers=None):
+    linked_layers = {}
+    target_image, _ = image_tag_from_url(target_image_url)
+    for layer in source_layers:
+        known_path, ref_image = image_utils.uploaded_layers_details(
+            uploaded_layers, layer, scope='local')
+
+        if layer not in image_layers and not ref_image:
+            continue
+
+        image_url = image_layers.get(layer, None)
+        if image_url:
+            image, _ = image_tag_from_url(image_url)
+        else:
+            image = ref_image
+        if not image:
+            continue
+
+        if known_path and ref_image:
+            blob_path = known_path
+            image = ref_image
+            if ref_image != image:
+                LOG.debug('[%s] Layer ref. by image %s already exists '
+                          'at %s' % (image, ref_image, known_path))
+            else:
+                LOG.debug('[%s] Layer already exists at %s'
+                          % (image, known_path))
+        else:
+            dir_path = os.path.join(IMAGE_EXPORT_DIR, 'v2', image, 'blobs')
+            blob_path = os.path.join(dir_path, '%s.gz' % layer)
+            if not os.path.exists(blob_path):
+                LOG.debug('[%s] Layer not found: %s' % (image, blob_path))
+                continue
+
+        layer_cross_link(layer, image, blob_path, target_image_url)
+        linked_layers.update({layer: {'known_path': blob_path,
+                                      'ref_image': image}})
+    return linked_layers
+
+
 def export_manifest_config(target_url,
                            manifest_str,
                            manifest_type,
-                           config_str):
+                           config_str,
+                           multi_arch=False):
     image, tag = image_tag_from_url(target_url)
     manifest = json.loads(manifest_str)
     if config_str is not None:
@@ -166,14 +229,18 @@ def export_manifest_config(target_url,
 
     manifests_path = os.path.join(
         IMAGE_EXPORT_DIR, 'v2', image, 'manifests')
+    manifests_htaccess_path = os.path.join(manifests_path, '.htaccess')
     manifest_dir_path = os.path.join(manifests_path, manifest_digest)
-    manifest_symlink_path = os.path.join(manifests_path, tag)
     manifest_path = os.path.join(manifest_dir_path, 'index.json')
     htaccess_path = os.path.join(manifest_dir_path, '.htaccess')
 
     make_dir(manifest_dir_path)
     build_catalog()
 
+    with open(manifests_htaccess_path, 'w+') as f:
+        f.write('AddHandler type-map %s\n' % TYPE_MAP_EXTENSION)
+        f.write('MultiviewsMatch Handlers\n')
+
     headers = collections.OrderedDict()
     headers['Content-Type'] = manifest_type
     headers['Docker-Content-Digest'] = manifest_digest
@@ -183,26 +250,92 @@ def export_manifest_config(target_url,
             f.write('Header set %s "%s"\n' % header)
 
     with open(manifest_path, 'w+b') as f:
-        f.write(manifest_str.encode('utf-8'))
+        manifest_data = manifest_str.encode('utf-8')
+        f.write(manifest_data)
 
-    if os.path.exists(manifest_symlink_path):
-        os.remove(manifest_symlink_path)
-    os.symlink(manifest_dir_path, manifest_symlink_path)
+    manifest_dict = {}
+    if multi_arch:
+        if manifest_type == MEDIA_MANIFEST_V2_LIST:
+            manifest_dict[manifest_type] = manifest_digest
+            # choose one of the entries to be the default v2 manifest
+            # to return:
+            # - If architecture amd64 exists, choose that
+            # - Otherwise choose the first entry
+            entries = manifest.get('manifests')
+            if entries:
+                entry = None
+                for i in entries:
+                    if i.get('platform', {}).get('architecture') == 'amd64':
+                        entry = i
+                        break
+                if not entry:
+                    entry = entries[0]
+                manifest_dict[entry['mediaType']] = entry['digest']
+
+    else:
+        manifest_dict[manifest_type] = manifest_digest
+
+    if manifest_dict:
+        write_type_map_file(image, tag, manifest_dict)
     build_tags_list(image)
 
 
+def write_type_map_file(image, tag, manifest_dict):
+    manifests_path = os.path.join(
+        IMAGE_EXPORT_DIR, 'v2', image, 'manifests')
+    type_map_path = os.path.join(manifests_path, '%s%s' %
+                                 (tag, TYPE_MAP_EXTENSION))
+    with open(type_map_path, 'w+') as f:
+        f.write('URI: %s\n\n' % tag)
+        for manifest_type, digest in manifest_dict.items():
+            f.write('Content-Type: %s\n' % manifest_type)
+            f.write('URI: %s/index.json\n\n' % digest)
+
+
+def parse_type_map_file(type_map_path):
+    uri = None
+    content_type = None
+    type_map = {}
+    with open(type_map_path, 'r') as f:
+        for l in f:
+            line = l[:-1]
+            if not line:
+                if uri and content_type:
+                    type_map[content_type] = uri
+                uri = None
+                content_type = None
+            else:
+                key, value = line.split(': ')
+                if key == TYPE_KEY_URI:
+                    uri = value
+                elif key == TYPE_KEY_TYPE:
+                    content_type = value
+    return type_map
+
+
+def migrate_to_type_map_file(image, manifest_symlink_path):
+    tag = os.path.split(manifest_symlink_path)[-1]
+    manifest_dir = os.readlink(manifest_symlink_path)
+    manifest_digest = os.path.split(manifest_dir)[-1]
+    write_type_map_file(image, tag, {MEDIA_MANIFEST_V2: manifest_digest})
+    os.remove(manifest_symlink_path)
+
+
 def build_tags_list(image):
     manifests_path = os.path.join(
         IMAGE_EXPORT_DIR, 'v2', image, 'manifests')
     tags_dir_path = os.path.join(IMAGE_EXPORT_DIR, 'v2', image, 'tags')
     tags_list_path = os.path.join(tags_dir_path, 'list')
-    LOG.debug('Rebuilding %s' % tags_dir_path)
+    LOG.debug('[%s] Rebuilding %s' % (image, tags_dir_path))
     make_dir(tags_dir_path)
     tags = []
     for f in os.listdir(manifests_path):
         f_path = os.path.join(manifests_path, f)
         if os.path.islink(f_path):
             tags.append(f)
+            migrate_to_type_map_file(image, f_path)
+        if f.endswith(TYPE_MAP_EXTENSION):
+            tags.append(f[:-len(TYPE_MAP_EXTENSION)])
 
     tags_data = {
         "name": image,
@@ -234,11 +367,19 @@ def delete_image(image_url):
     image, tag = image_tag_from_url(image_url)
     manifests_path = os.path.join(
         IMAGE_EXPORT_DIR, 'v2', image, 'manifests')
-    manifest_symlink_path = os.path.join(manifests_path, tag)
 
-    # delete manifest_symlink_path
-    LOG.debug('Deleting tag symlink %s' % manifest_symlink_path)
-    os.remove(manifest_symlink_path)
+    manifest_symlink_path = os.path.join(manifests_path, tag)
+    if os.path.exists(manifest_symlink_path):
+        LOG.debug('[%s] Deleting legacy tag symlink %s' %
+                  (image, manifest_symlink_path))
+        os.remove(manifest_symlink_path)
+
+    type_map_path = os.path.join(manifests_path, '%s%s' %
+                                 (tag, TYPE_MAP_EXTENSION))
+    if os.path.exists(type_map_path):
+        LOG.debug('[%s] Deleting typemap file %s' % (image, type_map_path))
+        os.remove(type_map_path)
+
     build_tags_list(image)
 
     # build list of manifest_dir_path without symlinks
@@ -246,8 +387,11 @@ def delete_image(image_url):
     manifest_dirs = set()
     for f in os.listdir(manifests_path):
         f_path = os.path.join(manifests_path, f)
-        if os.path.islink(f_path):
-            linked_manifest_dirs.add(os.readlink(f_path))
+        if f_path.endswith(TYPE_MAP_EXTENSION):
+            for uri in parse_type_map_file(f_path).values():
+                linked_manifest_dir = os.path.dirname(
+                    os.path.join(manifests_path, uri))
+                linked_manifest_dirs.add(linked_manifest_dir)
         elif os.path.isdir(f_path):
             manifest_dirs.add(f_path)
 
@@ -255,7 +399,7 @@ def delete_image(image_url):
 
     # delete list of manifest_dir_path without symlinks
     for manifest_dir in delete_manifest_dirs:
-        LOG.debug('Deleting manifest %s' % manifest_dir)
+        LOG.debug('[%s] Deleting manifest %s' % (image, manifest_dir))
         shutil.rmtree(manifest_dir)
 
     # load all remaining manifests and build the set of of in-use blobs,
@@ -289,13 +433,14 @@ def delete_image(image_url):
                      for b in os.listdir(blobs_path)])
     delete_blobs = all_blobs.difference(reffed_blobs)
     for blob in delete_blobs:
-        LOG.debug('Deleting layer blob %s' % blob)
+        LOG.debug('[%s] Deleting layer blob %s' % (image, blob))
         os.remove(blob)
 
     # if no files left in manifests_path, delete the whole image
-    if not os.listdir(manifests_path):
+    remaining = os.listdir(manifests_path)
+    if not remaining or remaining == ['.htaccess']:
         image_path = os.path.join(IMAGE_EXPORT_DIR, 'v2', image)
-        LOG.debug('Deleting image directory %s' % image_path)
+        LOG.debug('[%s] Deleting image directory %s' % (image, image_path))
         shutil.rmtree(image_path)
 
     # rebuild the catalog for the current image list
