@@ -416,11 +416,13 @@ class TestBaseImageUploader(base.TestCase):
 
     @mock.patch('concurrent.futures.ThreadPoolExecutor')
     def test_discover_image_tags(self, mock_pool):
-        mock_pool.return_value.map.return_value = (
+        mock_map = mock.Mock()
+        mock_map.return_value = (
             ('docker.io/t/foo', 'a'),
             ('docker.io/t/bar', 'b'),
             ('docker.io/t/baz', 'c')
         )
+        mock_pool.return_value.__enter__.return_value.map = mock_map
         images = [
             'docker.io/t/foo',
             'docker.io/t/bar',
@@ -434,7 +436,7 @@ class TestBaseImageUploader(base.TestCase):
             },
             self.uploader.discover_image_tags(images, 'rdo_release')
         )
-        mock_pool.return_value.map.assert_called_once_with(
+        mock_map.assert_called_once_with(
             image_uploader.discover_tag_from_inspect,
             [
                 (self.uploader, 'docker.io/t/foo', 'rdo_release'),
@@ -769,12 +771,14 @@ class TestBaseImageUploader(base.TestCase):
 
     @mock.patch('concurrent.futures.ThreadPoolExecutor')
     def test_list(self, mock_pool):
-        mock_pool.return_value.map.return_value = (
+        mock_map = mock.Mock()
+        mock_map.return_value = (
             ('localhost:8787/t/foo', ['a']),
             ('localhost:8787/t/bar', ['b']),
             ('localhost:8787/t/baz', ['c', 'd']),
             ('localhost:8787/t/bink', [])
         )
+        mock_pool.return_value.__enter__.return_value.map = mock_map
         session = mock.Mock()
         response = mock.Mock()
         response.status_code = 200
@@ -791,7 +795,7 @@ class TestBaseImageUploader(base.TestCase):
             ],
             self.uploader.list('localhost:8787', session=session)
         )
-        mock_pool.return_value.map.assert_called_once_with(
+        mock_map.assert_called_once_with(
             image_uploader.tags_for_image,
             [
                 (self.uploader, 'localhost:8787/t/foo', session),
@@ -800,14 +804,12 @@ class TestBaseImageUploader(base.TestCase):
                 (self.uploader, 'localhost:8787/t/bink', session)
             ])
 
-    @mock.patch('concurrent.futures.ThreadPoolExecutor')
-    def test_list_404(self, mock_pool):
+    def test_list_404(self):
         # setup bits
         session = mock.Mock()
         response = mock.Mock()
         response.status_code = 404
         session.get.return_value = response
-        mock_pool.return_value.map.return_value = ()
         # execute function
         return_val = self.uploader.list('localhost:8787', session=session)
         # check status of things
@@ -904,7 +906,8 @@ class TestSkopeoImageUploader(base.TestCase):
                 None,
                 None,
                 False,
-                'full')
+                'full',
+                False)
             )
         )
         mock_popen.assert_called_once_with([
@@ -954,7 +957,8 @@ class TestSkopeoImageUploader(base.TestCase):
                     'container_build_tool': 'buildah'
                 }
             }],
-            'hosts': 'localhost'
+            'hosts': 'localhost',
+            'gather_facts': 'no'
         }]
 
         # test response for a partial cleanup
@@ -968,7 +972,8 @@ class TestSkopeoImageUploader(base.TestCase):
                 'add-foo-plugin',
                 {'foo_version': '1.0.1'},
                 False,
-                'partial')
+                'partial',
+                False)
             )
         )
 
@@ -1028,7 +1033,7 @@ class TestSkopeoImageUploader(base.TestCase):
             self.uploader.upload_image, image_uploader.UploadTask(
                 image + ':' + tag, None, push_destination,
                 append_tag, 'add-foo-plugin', {'foo_version': '1.0.1'},
-                False, 'full')
+                False, 'full', False)
         )
 
         mock_copy.assert_called_once_with(
@@ -1056,7 +1061,8 @@ class TestSkopeoImageUploader(base.TestCase):
             'add-foo-plugin',
             {'foo_version': '1.0.1'},
             True,
-            'full')
+            'full',
+            False)
         )
 
         mock_ansible.assert_not_called()
@@ -1086,7 +1092,8 @@ class TestSkopeoImageUploader(base.TestCase):
             'add-foo-plugin',
             {'foo_version': '1.0.1'},
             False,
-            'full')
+            'full',
+            False)
         )
 
         mock_ansible.assert_not_called()
@@ -1174,6 +1181,8 @@ class TestPythonImageUploader(base.TestCase):
             source_session
         ]
         manifest = json.dumps({
+            'schemaVersion': 2,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2,
             'config': {
                 'digest': 'sha256:1234',
             },
@@ -1197,7 +1206,8 @@ class TestPythonImageUploader(base.TestCase):
             modify_role=None,
             modify_vars=None,
             dry_run=False,
-            cleanup='full'
+            cleanup='full',
+            multi_arch=False
         )
 
         self.assertEqual(
@@ -1223,7 +1233,7 @@ class TestPythonImageUploader(base.TestCase):
         ])
 
         _fetch_manifest.assert_called_once_with(
-            source_url, session=source_session)
+            source_url, session=source_session, multi_arch=False)
 
         _cross_repo_mount.assert_called_once_with(
             target_url,
@@ -1238,9 +1248,11 @@ class TestPythonImageUploader(base.TestCase):
         _copy_registry_to_registry.assert_called_once_with(
             source_url,
             target_url,
-            source_manifest=manifest,
+            source_manifests=[manifest],
             source_session=source_session,
-            target_session=target_session
+            target_session=target_session,
+            source_layers=['sha256:aaa', 'sha256:bbb', 'sha256:ccc'],
+            multi_arch=False
         )
 
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1291,7 +1303,8 @@ class TestPythonImageUploader(base.TestCase):
             modify_role=None,
             modify_vars=None,
             dry_run=False,
-            cleanup='full'
+            cleanup='full',
+            multi_arch=False
         )
 
         self.assertEqual(
@@ -1359,7 +1372,8 @@ class TestPythonImageUploader(base.TestCase):
             modify_role=None,
             modify_vars=None,
             dry_run=False,
-            cleanup='full'
+            cleanup='full',
+            multi_arch=False
         )
 
         self.assertEqual(
@@ -1426,7 +1440,8 @@ class TestPythonImageUploader(base.TestCase):
             modify_role=None,
             modify_vars=None,
             dry_run=False,
-            cleanup='full'
+            cleanup='full',
+            multi_arch=False
         )
 
         self.assertEqual(
@@ -1452,7 +1467,7 @@ class TestPythonImageUploader(base.TestCase):
         ])
 
         _fetch_manifest.assert_called_once_with(
-            source_url, session=source_session)
+            source_url, session=source_session, multi_arch=False)
 
         _cross_repo_mount.assert_called_once_with(
             target_url,
@@ -1467,9 +1482,11 @@ class TestPythonImageUploader(base.TestCase):
         _copy_registry_to_registry.assert_called_once_with(
             source_url,
             target_url,
-            source_manifest=manifest,
+            source_manifests=[manifest],
             source_session=source_session,
-            target_session=target_session
+            target_session=target_session,
+            source_layers=['sha256:aaa', 'sha256:bbb', 'sha256:ccc'],
+            multi_arch=False
         )
 
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1504,6 +1521,8 @@ class TestPythonImageUploader(base.TestCase):
             source_session
         ]
         manifest = json.dumps({
+            'schemaVersion': 2,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2,
             'config': {
                 'digest': 'sha256:1234',
             },
@@ -1528,7 +1547,8 @@ class TestPythonImageUploader(base.TestCase):
             modify_role='add-foo-plugin',
             modify_vars={'foo_version': '1.0.1'},
             dry_run=False,
-            cleanup='full'
+            cleanup='full',
+            multi_arch=False
         )
 
         source_url = urlparse(
@@ -1565,7 +1585,7 @@ class TestPythonImageUploader(base.TestCase):
         ])
 
         _fetch_manifest.assert_called_once_with(
-            source_url, session=source_session)
+            source_url, session=source_session, multi_arch=False)
 
         _cross_repo_mount.assert_has_calls([
             mock.call(
@@ -1593,9 +1613,11 @@ class TestPythonImageUploader(base.TestCase):
         _copy_registry_to_registry.assert_called_once_with(
             source_url,
             unmodified_target_url,
-            source_manifest=manifest,
+            source_manifests=[manifest],
             source_session=source_session,
-            target_session=target_session
+            target_session=target_session,
+            source_layers=['sha256:aaa', 'sha256:bbb', 'sha256:ccc'],
+            multi_arch=False
         )
         _copy_registry_to_local.assert_called_once_with(unmodified_target_url)
         run_modify_playbook.assert_called_once_with(
@@ -1614,6 +1636,57 @@ class TestPythonImageUploader(base.TestCase):
             session=target_session
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._detect_target_export')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader.credentials_for_registry')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._copy_local_to_registry')
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader.authenticate')
+    def test_upload_image_local(self, authenticate, mock_copy, mock_creds,
+                                mock_detect):
+
+        mock_creds.return_value = (None, None)
+        target_session = mock.Mock()
+        authenticate.side_effect = [
+            target_session
+        ]
+
+        image = 'docker.io/tripleomaster/heat-docker-agents-centos'
+        tag = 'latest'
+        push_destination = 'localhost:8787'
+        source_image = 'containers-storage:%s:%s' % (image, tag)
+        task = image_uploader.UploadTask(
+            image_name=source_image,
+            pull_source=None,
+            push_destination=push_destination,
+            append_tag=None,
+            modify_role=None,
+            modify_vars=None,
+            dry_run=False,
+            cleanup='full',
+            multi_arch=False
+        )
+
+        self.assertEqual(
+            [],
+            self.uploader.upload_image(task)
+        )
+        source_url = urlparse(source_image)
+        target_url = urlparse('docker://localhost:8787/tripleomaster/'
+                              'heat-docker-agents-centos:latest')
+        authenticate.assert_has_calls([
+            mock.call(
+                target_url,
+                username=None,
+                password=None
+            )
+        ])
+        mock_detect.assert_called_once_with(target_url, target_session)
+        mock_copy.assert_called_once_with(source_url, target_url,
+                                          session=target_session)
+
     @mock.patch('tripleo_common.image.image_uploader.'
                 'BaseImageUploader.check_status')
     def test_fetch_manifest(self, check_status):
@@ -1623,7 +1696,7 @@ class TestPythonImageUploader(base.TestCase):
         session.get.return_value.text = manifest
         self.assertEqual(
             manifest,
-            self.uploader._fetch_manifest(url, session)
+            self.uploader._fetch_manifest(url, session, multi_arch=False)
         )
 
         session.get.assert_called_once_with(
@@ -1675,15 +1748,11 @@ class TestPythonImageUploader(base.TestCase):
 
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._upload_url')
-    def test_copy_layer_registry_to_registry(self, _upload_url):
+    @mock.patch('tripleo_common.utils.image.uploaded_layers_details')
+    def test_copy_layer_registry_to_registry(self, global_check, _upload_url):
         _upload_url.return_value = 'https://192.168.2.1:5000/v2/upload'
         source_url = urlparse('docker://docker.io/t/nova-api:latest')
         target_url = urlparse('docker://192.168.2.1:5000/t/nova-api:latest')
-        layer = {
-            'digest': 'sha256:aaaa',
-            'size': 8,
-            'mediaType': 'application/vnd.docker.image.rootfs.diff.tar'
-        }
         source_session = requests.Session()
         target_session = requests.Session()
 
@@ -1691,10 +1760,17 @@ class TestPythonImageUploader(base.TestCase):
         calc_digest = hashlib.sha256()
         calc_digest.update(blob_data)
         blob_digest = 'sha256:' + calc_digest.hexdigest()
+        layer_entry = {
+            'digest': blob_digest,
+            'size': 8,
+            'mediaType': 'application/vnd.docker.image.rootfs.diff.tar.gzip'
+        }
+        layer = layer_entry['digest']
 
         # layer already exists at destination
+        global_check.return_value = (None, None)
         self.requests.head(
-            'https://192.168.2.1:5000/v2/t/nova-api/blobs/sha256:aaaa',
+            'https://192.168.2.1:5000/v2/t/nova-api/blobs/%s' % blob_digest,
             status_code=200
         )
         self.assertIsNone(
@@ -1709,7 +1785,7 @@ class TestPythonImageUploader(base.TestCase):
 
         # layer needs transferring
         self.requests.head(
-            'https://192.168.2.1:5000/v2/t/nova-api/blobs/sha256:aaaa',
+            'https://192.168.2.1:5000/v2/t/nova-api/blobs/%s' % blob_digest,
             status_code=404
         )
         self.requests.put(
@@ -1719,7 +1795,8 @@ class TestPythonImageUploader(base.TestCase):
             'https://192.168.2.1:5000/v2/upload',
         )
         self.requests.get(
-            'https://registry-1.docker.io/v2/t/nova-api/blobs/sha256:aaaa',
+            'https://registry-1.docker.io/v2/t/nova-api/blobs/%s' %
+            blob_digest,
             content=blob_data
         )
 
@@ -1740,7 +1817,7 @@ class TestPythonImageUploader(base.TestCase):
                              'vnd.docker.image.rootfs.diff.tar.gzip',
                 'size': 8
             },
-            layer
+            layer_entry
         )
 
     def test_assert_scheme(self):
@@ -1778,8 +1855,11 @@ class TestPythonImageUploader(base.TestCase):
         source_session.get.return_value.text = '{}'
 
         manifest = json.dumps({
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2,
             'config': {
-                'digest': 'sha256:1234'
+                'digest': 'sha256:1234',
+                'size': 2,
+                'mediaType': image_uploader.MEDIA_CONFIG
             },
             'layers': [
                 {'digest': 'sha256:aaaa'},
@@ -1792,7 +1872,7 @@ class TestPythonImageUploader(base.TestCase):
         ]
 
         self.uploader._copy_registry_to_registry(
-            source_url, target_url, manifest,
+            source_url, target_url, [manifest],
             source_session=source_session,
             target_session=target_session
         )
@@ -1876,8 +1956,9 @@ class TestPythonImageUploader(base.TestCase):
     @mock.patch('subprocess.Popen')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._upload_url')
-    def test_copy_layer_local_to_registry(self, _upload_url, mock_popen,
-                                          mock_exists):
+    @mock.patch('tripleo_common.utils.image.uploaded_layers_details')
+    def test_copy_layer_local_to_registry(self, global_check, _upload_url,
+                                          mock_popen, mock_exists):
         mock_exists.return_value = True
         _upload_url.return_value = 'https://192.168.2.1:5000/v2/upload'
         target_url = urlparse('docker://192.168.2.1:5000/t/nova-api:latest')
@@ -1902,6 +1983,7 @@ class TestPythonImageUploader(base.TestCase):
         }
 
         # layer already exists at destination
+        global_check.return_value = (None, None)
         self.requests.head(
             'https://192.168.2.1:5000/v2/t/'
             'nova-api/blobs/%s' % compressed_digest,
@@ -1971,6 +2053,7 @@ class TestPythonImageUploader(base.TestCase):
             layer
         )
 
+    @mock.patch('tripleo_common.utils.image.uploaded_layers_details')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._image_manifest_config')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1981,11 +2064,12 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._upload_url')
     def test_copy_local_to_registry(self, _upload_url, _containers_json,
                                     _copy_layer_local_to_registry,
-                                    _image_manifest_config):
+                                    _image_manifest_config, _global_check):
         source_url = urlparse('containers-storage:/t/nova-api:latest')
         target_url = urlparse('docker://192.168.2.1:5000/t/nova-api:latest')
         target_session = requests.Session()
         _upload_url.return_value = 'https://192.168.2.1:5000/v2/upload'
+        _global_check.return_value = (None, None)
         layers = [{
             "compressed-diff-digest": "sha256:aeb786",
             "compressed-size": 74703002,
@@ -2230,3 +2314,154 @@ class TestPythonImageUploader(base.TestCase):
             mock.call(urlparse('containers-storage:baz')),
             mock.call(urlparse('containers-storage:foo'))
         ])
+
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._fetch_manifest')
+    def test_collect_manifests_layers(self, _fetch_manifest):
+        manifest = {
+            'schemaVersion': 2,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2,
+            'config': {
+                'mediaType': image_uploader.MEDIA_CONFIG,
+                'digest': 'sha256:1111'
+            },
+            'layers': [
+                {'digest': 'sha256:2222'},
+                {'digest': 'sha256:3333'},
+                {'digest': 'sha256:4444'}
+            ]
+        }
+        manifest_str = json.dumps(manifest, indent=2)
+        _fetch_manifest.return_value = manifest_str
+        url = urlparse('docker://docker.io/t/nova-api:latest')
+        session = requests.Session()
+        layers = []
+        manifests_str = []
+
+        self.uploader._collect_manifests_layers(
+            url, session, manifests_str, layers, False)
+        self.assertEqual([manifest_str], manifests_str)
+        self.assertEqual(
+            [
+                'sha256:2222',
+                'sha256:3333',
+                'sha256:4444',
+            ],
+            layers
+        )
+
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._fetch_manifest')
+    def test_collect_manifests_layers_v1(self, _fetch_manifest):
+        manifest = {
+            'schemaVersion': 1,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V1,
+            'fsLayers': [
+                {'blobSum': 'sha256:4444'},
+                {'blobSum': 'sha256:3333'},
+                {'blobSum': 'sha256:2222'},
+            ]
+        }
+        manifest_str = json.dumps(manifest, indent=2)
+        _fetch_manifest.return_value = manifest_str
+        url = urlparse('docker://docker.io/t/nova-api:latest')
+        session = requests.Session()
+        layers = []
+        manifests_str = []
+
+        self.uploader._collect_manifests_layers(
+            url, session, manifests_str, layers, False)
+        self.assertEqual([manifest_str], manifests_str)
+        self.assertEqual(
+            [
+                'sha256:2222',
+                'sha256:3333',
+                'sha256:4444',
+            ],
+            layers
+        )
+
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'PythonImageUploader._fetch_manifest')
+    def test_collect_manifests_layers_multi_arch(self, _fetch_manifest):
+        manifest_x86 = {
+            'schemaVersion': 2,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2,
+            'config': {
+                'mediaType': image_uploader.MEDIA_CONFIG,
+                'digest': 'sha256:1111'
+            },
+            'layers': [
+                {'digest': 'sha256:2222'},
+                {'digest': 'sha256:3333'},
+                {'digest': 'sha256:4444'}
+            ]
+        }
+        manifest_ppc = {
+            'schemaVersion': 2,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2,
+            'config': {
+                'mediaType': image_uploader.MEDIA_CONFIG,
+                'digest': 'sha256:5555'
+            },
+            'layers': [
+                {'digest': 'sha256:6666'},
+                {'digest': 'sha256:7777'},
+                {'digest': 'sha256:8888'}
+            ]
+        }
+        manifest = {
+            'schemaVersion': 2,
+            'mediaType': image_uploader.MEDIA_MANIFEST_V2_LIST,
+            "manifests": [
+                {
+                    "mediaType": image_uploader.MEDIA_MANIFEST_V2,
+                    "digest": "sha256:bbbb",
+                    "platform": {
+                        "architecture": "amd64",
+                        "os": "linux",
+                        "features": ["sse4"]
+                    }
+                },
+                {
+                    "mediaType": image_uploader.MEDIA_MANIFEST_V2,
+                    "digest": "sha256:aaaa",
+                    "platform": {
+                        "architecture": "ppc64le",
+                        "os": "linux",
+                    }
+                }
+            ]
+        }
+        manifest_str = json.dumps(manifest, indent=2)
+        _fetch_manifest.side_effect = [
+            manifest_str,
+            json.dumps(manifest_x86),
+            json.dumps(manifest_ppc)
+        ]
+        url = urlparse('docker://docker.io/t/nova-api:latest')
+        session = requests.Session()
+        layers = []
+        manifests_str = []
+
+        self.uploader._collect_manifests_layers(
+            url, session, manifests_str, layers, multi_arch=True)
+        self.assertEqual(
+            [
+                manifest_str,
+                json.dumps(manifest_x86),
+                json.dumps(manifest_ppc)
+            ],
+            manifests_str
+        )
+        self.assertEqual(
+            [
+                'sha256:2222',
+                'sha256:3333',
+                'sha256:4444',
+                'sha256:6666',
+                'sha256:7777',
+                'sha256:8888',
+            ],
+            layers
+        )
