@@ -236,7 +236,7 @@ class TestKollaImageBuilderTemplate(base.TestCase):
                 'namespace': 'docker.io/tripleostein',
                 'ceph_namespace': 'docker.io/ceph',
                 'ceph_image': 'daemon',
-                'ceph_tag': 'v4.0.1-stable-4.0-nautilus-centos-7-x86_64',
+                'ceph_tag': 'v4.0.4-stable-4.0-nautilus-centos-7-x86_64',
                 'name_prefix': 'centos-binary-',
                 'name_suffix': '',
                 'tag': 'current-tripleo',
@@ -1026,6 +1026,7 @@ class TestPrepare(base.TestCase):
     @mock.patch('tripleo_common.image.image_uploader.ImageUploadManager',
                 autospec=True)
     def test_container_images_prepare_multi(self, mock_im, mock_cip):
+        mock_lock = mock.MagicMock()
         mapping_args = {
             'namespace': 't',
             'name_prefix': '',
@@ -1079,7 +1080,8 @@ class TestPrepare(base.TestCase):
             },
         ]
 
-        image_params = kb.container_images_prepare_multi(env, roles_data)
+        image_params = kb.container_images_prepare_multi(env, roles_data,
+                                                         lock=mock_lock)
 
         mock_cip.assert_has_calls([
             mock.call(
@@ -1101,7 +1103,9 @@ class TestPrepare(base.TestCase):
                 },
                 registry_credentials={
                     'docker.io': {'my_username': 'my_password'}
-                }
+                },
+                multi_arch=False,
+                lock=mock_lock
             ),
             mock.call(
                 excludes=['nova', 'neutron'],
@@ -1122,7 +1126,9 @@ class TestPrepare(base.TestCase):
                 },
                 registry_credentials={
                     'docker.io': {'my_username': 'my_password'}
-                }
+                },
+                multi_arch=False,
+                lock=mock_lock
             )
         ])
 
@@ -1142,6 +1148,7 @@ class TestPrepare(base.TestCase):
     @mock.patch('tripleo_common.image.image_uploader.ImageUploadManager',
                 autospec=True)
     def test_container_images_prepare_multi_dry_run(self, mock_im, mock_cip):
+        mock_lock = mock.MagicMock()
         mapping_args = {
             'namespace': 't',
             'name_prefix': '',
@@ -1190,7 +1197,8 @@ class TestPrepare(base.TestCase):
             },
         ]
 
-        image_params = kb.container_images_prepare_multi(env, roles_data, True)
+        image_params = kb.container_images_prepare_multi(env, roles_data, True,
+                                                         lock=mock_lock)
 
         mock_cip.assert_has_calls([
             mock.call(
@@ -1208,7 +1216,9 @@ class TestPrepare(base.TestCase):
                 modify_only_with_labels=None,
                 modify_vars=None,
                 mirrors={},
-                registry_credentials=None
+                registry_credentials=None,
+                multi_arch=False,
+                lock=mock_lock
             ),
             mock.call(
                 excludes=['nova', 'neutron'],
@@ -1225,12 +1235,15 @@ class TestPrepare(base.TestCase):
                 modify_only_with_labels=['kolla_version'],
                 modify_vars={'foo_version': '1.0.1'},
                 mirrors={},
-                registry_credentials=None
+                registry_credentials=None,
+                multi_arch=False,
+                lock=mock_lock
             )
         ])
 
         mock_im.assert_called_once_with(mock.ANY, dry_run=True, cleanup='full',
-                                        mirrors={}, registry_credentials=None)
+                                        mirrors={}, registry_credentials=None,
+                                        multi_arch=False, lock=mock_lock)
 
         self.assertEqual(
             {
