@@ -389,10 +389,15 @@ def register_ironic_node(node, client):
         extra = dict(tripleo_platform=platform)
 
     if 'capabilities' in node:
-        caps = node['capabilities']
-        if isinstance(caps, dict):
-            caps = dict_to_capabilities(caps)
-        properties.update({"capabilities": six.text_type(caps)})
+        caps = capabilities_to_dict(node['capabilities'])
+    else:
+        caps = {}
+
+    if 'profile' in node:
+        caps['profile'] = node['profile']
+
+    if caps:
+        properties["capabilities"] = dict_to_capabilities(caps)
 
     driver = node['pm_type']
     if handler.hardware_type and handler.hardware_type != driver:
@@ -490,7 +495,6 @@ _NON_DRIVER_FIELDS = {'cpu': '/properties/cpus',
                                     '/driver_info/rescue_kernel'],
                       'ramdisk_id': ['/driver_info/deploy_ramdisk',
                                      '/driver_info/rescue_ramdisk'],
-                      'capabilities': '/properties/capabilities',
                       'platform': '/extra/tripleo_platform',
                       }
 
@@ -516,6 +520,17 @@ def _update_or_register_ironic_node(node, node_map, client):
                 for path in paths:
                     patched[path] = value
 
+        if 'capabilities' in node:
+            caps = capabilities_to_dict(node.pop('capabilities'))
+        else:
+            caps = {}
+
+        if 'profile' in node:
+            caps['profile'] = node.pop('profile')
+
+        if caps:
+            patched['/properties/capabilities'] = dict_to_capabilities(caps)
+
         driver_info = handler.convert(node)
         for key, value in driver_info.items():
             patched['/driver_info/%s' % key] = value
@@ -581,6 +596,10 @@ def register_all_nodes(nodes_list, client, remove=False, glance_client=None,
     return seen
 
 
+# These fields are treated specially during enrolling/updating
+_SPECIAL_NON_DRIVER_FIELDS = {'ports', 'pm_type', 'capabilities'}
+
+
 def validate_nodes(nodes_list):
     """Validate all nodes list.
 
@@ -650,7 +669,7 @@ def validate_nodes(nodes_list):
         for field in node:
             converted = handler.convert_key(field)
             if (converted is None and field not in _NON_DRIVER_FIELDS and
-                    field not in ('ports', 'pm_type')):
+                    field not in _SPECIAL_NON_DRIVER_FIELDS):
                 failures.append((index, 'Unknown field %s' % field))
 
     if failures:
@@ -660,8 +679,14 @@ def validate_nodes(nodes_list):
 
 def dict_to_capabilities(caps_dict):
     """Convert a dictionary into a string with the capabilities syntax."""
+    if isinstance(caps_dict, six.string_types):
+        return caps_dict
+
+    # NOTE(dtantsur): sort capabilities so that their order does not change
+    # between updates.
+    items = sorted(caps_dict.items(), key=lambda tpl: tpl[0])
     return ','.join(["%s:%s" % (key, value)
-                     for key, value in caps_dict.items()
+                     for key, value in items
                      if value is not None])
 
 
